package net.sf.jdatabaseimport;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Represents a database table.
 * 
 * @author Guilherme Silveira
 * 
 */
public class Table {

	private static final Log LOG = LogFactory.getLog(Exporter.class);

	private final String name;

	private final Set<String> ignore = new HashSet<String>();

	private final Map<String, String> columnAliases = new HashMap<String, String>();

	public Table(final String name) {
		this.name = name;
	}

	public void ignoreField(String field) {
		this.ignore.add(field);
	}

	public void aliasField(String field, String alias) {
		this.columnAliases.put(field, alias);
	}

	public void clear(Connection con) throws SQLException {
		LOG.debug("Deleting everything from " + name);
		execute(con, "delete from " + name);
	}

	private void execute(Connection con, String sql) throws SQLException {
		PreparedStatement statement = con.prepareStatement(sql);
		statement.execute();
		statement.close();
	}

	public void send(Connection from, Connection to) throws SQLException {

		System.out.println("Filling table " + name);

		PreparedStatement stmt = from.prepareStatement("select * from " + name);
		ResultSet rs = stmt.executeQuery();
		ResultSetMetaData metaData = rs.getMetaData();
		int columnCount = metaData.getColumnCount();
		int fields[] = new int[columnCount];
		String columns = "";
		int ignored = 0;
		String values = "";
		for (int i = 1; i <= columnCount; i++) {
			String realName = metaData.getColumnName(i);
			if (ignore.contains(realName)) {
				LOG.info("Ignoring " + realName + " on " + name);
				ignored++;
				continue;
			}
			fields[i - ignored - 1] = i;
			columns += (columns.length() == 0 ? "" : ",")
					+ getTranslatedColumnName(realName);
			values += (values.length() == 0 ? "?" : ",?");
		}

		columnCount -= ignored;

		for (int i = 1; i <= columnCount; i++) {
			LOG.debug("Field " + (i - 1) + " is "
					+ metaData.getColumnName(fields[i - 1]) + "("
					+ fields[i - 1] + ")");
		}

		String insertSql = "insert into " + name + " (" + columns
				+ ") values (" + values + ")";
		LOG.info(insertSql);
		PreparedStatement insert = to.prepareStatement(insertSql);
		int total = 0;
		Object[] params = new Object[columnCount];
		while (rs.next()) {
			insert.clearParameters();
			for (int i = 1; i <= columnCount; i++) {
				params[i - 1] = setField(insert, fields[i - 1], i, rs, metaData);
			}
			try {
				insert.execute();
			} catch (SQLException ex) {
				if (ex.getErrorCode() == 1054) {
					System.err
							.println("A column was not found in the target database, it should be ignored.");
				}
				LOG.error("Unable to save params: " + Arrays.toString(params));
				throw ex;
			}
			total++;
		}
		System.out.println("\t" + total);
		insert.close();
		rs.close();
	}

	private Object setField(PreparedStatement insert, int columnFrom,
			int columnTo, ResultSet rs, ResultSetMetaData metaData)
			throws SQLException {
		String type = metaData.getColumnTypeName(columnFrom);
		if (type.equals("VARCHAR")) {
			insert.setString(columnTo, rs.getString(columnFrom));
		} else if (type.equals("BIGINT")) {
			insert.setLong(columnTo, rs.getLong(columnFrom));
		} else if (type.equals("DOUBLE")) {
			insert.setDouble(columnTo, rs.getDouble(columnFrom));
		} else if (type.equals("INTEGER")) {
			insert.setInt(columnTo, rs.getInt(columnFrom));
		} else if (type.equals("BOOLEAN")) {
			insert.setBoolean(columnTo, rs.getBoolean(columnFrom));
		} else if (type.equals("TIMESTAMP")) {
			insert.setTimestamp(columnTo, rs.getTimestamp(columnFrom));
		} else {
			throw new RuntimeException("Unknown column type " + type);
		}
		return rs.getObject(columnFrom);
	}

	private String getTranslatedColumnName(String key) {
		if (columnAliases.containsKey(key)) {
			return columnAliases.get(key);
		}
		return key;
	}

}
